-- Display filter -Y doesn't interact with Lua
-- local wlan = Listener.new("wlan", "wlan.fcs.status == 1") -- Error due to String/Boolean/BASE_NONE ?
-- https://ask.wireshark.org/question/18311/lua-display-filter-in-listenernew/
-- Also bad_checksum is an expert field, present or not, not containing values
-- F    Frame check sequence    wlan.fcs    FT_UINT32    wlan    BASE_HEX    0x0    Frame Check Sequence (FCS)
-- F    FCS Status    wlan.fcs.status    FT_UINT8    wlan    BASE_NONE    0x0
-- F    Bad checksum    wlan.fcs.bad_checksum    FT_NONE    wlan        0x0

local wlan = Listener.new("wlan", "radiotap.flags.badfcs == 0")

local wifi_beacons = {}
local launch_time_epoch = 0
local time_epoch = 0

local frame_time_epoch_f        = Field.new("frame.time_epoch")

local rxbitmask_0to7_f          = Field.new("wlan.ht.mcsset.rxbitmask.0to7")
local rxbitmask_8to15_f         = Field.new("wlan.ht.mcsset.rxbitmask.8to15")
local rxbitmask_16to23_f        = Field.new("wlan.ht.mcsset.rxbitmask.16to23")
local rxbitmask_24to31_f        = Field.new("wlan.ht.mcsset.rxbitmask.24to31")

local wlan_fc_type_subtype_f          = Field.new("wlan.fc.type_subtype")
local wlan_ta_f                       = Field.new("wlan.ta")
local wlan_ssid_f                     = Field.new("wlan.ssid")
local wlan_country_info_code_f        = Field.new("wlan.country_info.code")
local radiotap_channel_freq_f         = Field.new("radiotap.channel.freq")
local radiotap_dbm_antsignal_f        = Field.new("radiotap.dbm_antsignal")

local wlan_ht_info_primarychannel_f = Field.new("wlan.ht.info.primarychannel")

-- Table 8-57 - 802.11-2012
-- 0 - no secondary channel
-- 1 - +1
-- 2 - Reserved
-- 3 - -1
local wlan_ht_info_secchanoffset_f  = Field.new("wlan.ht.info.secchanoffset")

-- Table 8-183x - 802.11ac-2013
-- 0 - 20 or 40 MHz
-- 1 - 80 MHz
-- 2 - 160 MHz
-- 3 - 80+80 MHz
local wlan_vht_op_channelwidth_f    = Field.new("wlan.vht.op.channelwidth")

-- WPA2 - PSK or 802.1X - Can be a list of several items - wlan.rsn.akms.list
local wlan_rsn_akms_type_f          = Field.new("wlan.rsn.akms.type")

-- WPA - PSK or 802.1X - Can be a list of several items - wlan.wfa.ie.wpa.akms.list
local wlan_wfa_ie_wpa_type_f        = Field.new("wlan.wfa.ie.wpa.type")

-- WPA2 - AES or TKIP - Can be a list of several items - wlan.rsn.pcs.list
local wlan_rsn_pcs_type_f           = Field.new("wlan.rsn.pcs.type")

-- WPA - AES or TKIP - Can be a list of several items - wlan.wfa.ie.wpa.ucs.list
local wlan_wfa_ie_wpa_ucs_type_f    = Field.new("wlan.wfa.ie.wpa.ucs.type")

local wlan_cisco_ccx1_name_f        = Field.new("wlan.cisco.ccx1.name")
local wlan_vs_aruba_ap_name_f        = Field.new("wlan.vs.aruba.ap_name")


local header_line = ""
local headers = {   "wlan.ta", 
                    "wlan.ssid",
                    "wlan.ssid_hex",
                    "wlan.country_info.code",
                    "radiotap.channel.freq",
                    "wlan.ht.info.primarychannel",
                    "wlan.ht.info.secchanoffset",
                    "wlan.vht.op.channelwidth",
                    "radiotap.dbm_antsignal",
                    "wlan.ht.mcsset.rxbitmask",
                    "wlan.akm.type.list",
                    "wlan.ciphersuite.list",
                    "wlan.cisco.ccx1.name",
                    "wlan.vs.aruba.ap_name"
                }


--for header = 1, #headers do
for i, header in ipairs(headers) do
    -- i starts at 1
    header_line = header_line .. header

    if i < #headers then
        header_line = header_line .. "\t"
    end
end

-- Display header in first line
print(header_line)

function addToStringWithDelim(bigString,tobeadded,delim)

    if tobeadded ~= '' then

        if #bigString == 0 then
            bigString = tobeadded
        else
            bigString = bigString .. delim ..tobeadded
        end
    end
    
    return bigString
end

function wlan.packet(pinfo,tvb,tapinfo)
    
    -- print("wlan.packet called")

    -- Must be a Beacon
    if (wlan_fc_type_subtype_f() ~= nil) then
        
        if (tostring(wlan_fc_type_subtype_f()) == "8") then
                
            local rxbitmask_0to7
            local rxbitmask_8to15
            local rxbitmask_16to23
            local rxbitmask_24to31
            local wlan_ht_mcsset_rxbitmask
    
            if (rxbitmask_0to7_f() ~= nil and 
                rxbitmask_8to15_f() ~= nil and
                rxbitmask_16to23_f() ~= nil and
                rxbitmask_24to31_f() ~= nil) then

                -- We can have several values for thoses fields, keep the first one
                -- print(rxbitmask_0to7_f())                        -- 0x000000ff	0x00000000
                -- print(tostring(rxbitmask_0to7_f()))              -- 0x000000ff
                -- print(tonumber(tostring(rxbitmask_0to7_f())))    -- 255
                rxbitmask_0to7 = tonumber(tostring(rxbitmask_0to7_f()))
                rxbitmask_8to15 = tonumber(tostring(rxbitmask_8to15_f()))
                rxbitmask_16to23 = tonumber(tostring(rxbitmask_16to23_f()))
                rxbitmask_24to31 = tonumber(tostring(rxbitmask_24to31_f()))
        
                local total_rxbitmask = rxbitmask_0to7 + rxbitmask_8to15 + rxbitmask_16to23 + rxbitmask_24to31
                wlan_ht_mcsset_rxbitmask = total_rxbitmask / 255
            else
                wlan_ht_mcsset_rxbitmask = 1
            end
    
    
            -- Must initialize to empty string, else nil value cut the final fields array
            local frame_time_epoch = ''
            local wlan_ta = ''
            local wlan_ssid = ''
            local wlan_ssid_hex = ''
            local wlan_country_info_code = ''
            local radiotap_channel_freq = ''
            local radiotap_dbm_antsignal = ''
            local wlan_ht_info_primarychannel = ''
            local wlan_ht_info_secchanoffset = ''
            local wlan_vht_op_channelwidth = ''
            local wlan_rsn_akms_type = ''
            local wlan_wfa_ie_wpa_type = ''
            local wlan_rsn_pcs_type = ''
            local wlan_wfa_ie_wpa_ucs_type = ''
            local wlan_cisco_ccx1_name = ''
            local wlan_vs_aruba_ap_name = ''
    
            -- print("Total: " .. total_rxbitmask)
            -- print("SSID: " .. tostring(wlan_ssid_f()) .. " - Number SS: " .. wlan_ht_mcsset_rxbitmask)

            if (frame_time_epoch_f() ~= nil) then
                -- Lua doesn't handle float, stripping chars to transform in integer
                -- 1452006449.297822000
                -- Changed to 2006449.297            
                frame_time_epoch = tostring(frame_time_epoch_f())
                frame_time_epoch = frame_time_epoch:gsub("^...", "")
                frame_time_epoch = frame_time_epoch:gsub("......$", "")
                frame_time_epoch = frame_time_epoch:gsub("%.", "")
                frame_time_epoch = tonumber(frame_time_epoch)

                if (time_epoch == 0) then
                    -- initialize
                    time_epoch = frame_time_epoch
                end
                
                if (launch_time_epoch == 0) then
                    launch_time_epoch = frame_time_epoch
                end
            end
            
    
            if (wlan_ta_f() ~= nil) then
                wlan_ta = tostring(wlan_ta_f())
            end
    
            local wifi_beacon = wifi_beacons[wlan_ta]
            
            if not wifi_beacon then
                if (wlan_ssid_f() ~= nil) then
                    
                    -- wlan_ssid = tostring(wlan_ssid_f())
                    -- wlan_ssid_hex = tostring(wlan_ssid_f().range)
                    
                    -- if SSID is longer than 24 chars, then the hex value is fixed to 24 + "..." due to tostring():
                    -- https://ask.wireshark.org/question/18387/lua-get-hex-value-of-a-field/
                    
                    local wlan_ssid_f = wlan_ssid_f()
                    
                    wlan_ssid = tostring(wlan_ssid_f)
                    
                    if wlan_ssid_f then
                        local wlan_ssid_tvb = wlan_ssid_f.range
                        if wlan_ssid_f.len <= 24 then
                            wlan_ssid_hex = tostring(wlan_ssid_tvb)
                        else
                            local offset = 0
                            local rem = wlan_ssid_f.len
                            while rem > 0 do
                                if rem > 24 then
                                    wlan_ssid_hex = wlan_ssid_hex .. tostring(wlan_ssid_tvb(offset, 24))
                                    rem = rem - 24
                                    offset = offset + 24
                                else
                                    wlan_ssid_hex = wlan_ssid_hex .. tostring(wlan_ssid_tvb(offset, rem))
                                    rem = 0
                                end
                            end
                        end
                    end
                end

                if (wlan_country_info_code_f() ~= nil) then
                    wlan_country_info_code = tostring(wlan_country_info_code_f())
                end
    
                if (radiotap_channel_freq_f() ~= nil) then
                    radiotap_channel_freq = tostring(radiotap_channel_freq_f())
                end
    
                if (radiotap_dbm_antsignal_f() ~= nil) then
                    radiotap_dbm_antsignal = tostring(radiotap_dbm_antsignal_f())
                end
            
                if (wlan_ht_info_primarychannel_f() ~= nil) then
                    wlan_ht_info_primarychannel = tostring(wlan_ht_info_primarychannel_f())
                end
            
                if (wlan_ht_info_secchanoffset_f() ~= nil) then
                    wlan_ht_info_secchanoffset = tostring(wlan_ht_info_secchanoffset_f())
                end

                if (wlan_vht_op_channelwidth_f() ~= nil) then
                    wlan_vht_op_channelwidth = tostring(wlan_vht_op_channelwidth_f())
                end
    
                -- WPA2 - PSK (2) or WPA (1)
                if (wlan_rsn_akms_type_f() ~= nil) then
                    local wlan_rsn_akms_type_list = { wlan_rsn_akms_type_f() }
                    for i, type_f in pairs(wlan_rsn_akms_type_list) do
           
                        if i > 1 then
                            wlan_rsn_akms_type = wlan_rsn_akms_type .. ","
                        end
            
                        local type_string
                        if (tostring(type_f())) == "0" then
                            type_string = "WPA2-None"
                        elseif (tostring(type_f())) == "1" then
                            type_string = "WPA2-802.1X"
                        elseif (tostring(type_f())) == "2" then
                                type_string = "WPA2-PSK"
                        elseif (tostring(type_f())) == "3" then
                                type_string = "WPA2-802.1X Fast Transition"
                        elseif (tostring(type_f())) == "4" then
                                type_string = "WPA2-PSK Fast Transition"
                        elseif (tostring(type_f())) == "5" then
                                type_string = "WPA2-802.1X (SHA-256)"
                        elseif (tostring(type_f())) == "6" then
                                type_string = "WPA2-PSK (SHA-256)"
                        elseif (tostring(type_f())) == "6" then
                                type_string = "WPA2-TDLS / TPK Handshake"
                        else
                            type_string = tostring(type_f())
                        end
            
                        wlan_rsn_akms_type = wlan_rsn_akms_type .. type_string
                    end
                end
    
                -- WPA - PSK (2) or WPA (1)
                if (wlan_wfa_ie_wpa_type_f() ~= nil) then
                    local wlan_wfa_ie_wpa_type_list = { wlan_wfa_ie_wpa_type_f() }
                    for i, type_f in pairs(wlan_wfa_ie_wpa_type_list) do
           
                        if i > 1 then
                            wlan_wfa_ie_wpa_type = wlan_wfa_ie_wpa_type .. ","
                        end
                   
                       local type_string
                       if (tostring(type_f())) == "0" then
                           type_string = "WPA-None"
                       elseif (tostring(type_f())) == "1" then
                           type_string = "WPA-802.1X"
                       elseif (tostring(type_f())) == "2" then
                           type_string = "WPA-PSK"
                       elseif (tostring(type_f())) == "3" then
                           type_string = "WPA-802.1X Fast Transition"
                       elseif (tostring(type_f())) == "4" then
                           type_string = "WPA-PSK Fast Transition"
                       elseif (tostring(type_f())) == "5" then
                           type_string = "WPA-802.1X (SHA-256)"
                       elseif (tostring(type_f())) == "6" then
                           type_string = "WPA-PSK (SHA-256)"
                       elseif (tostring(type_f())) == "6" then
                           type_string = "WPA-TDLS / TPK Handshake"
                       else
                           type_string = tostring(type_f())
                       end

                        wlan_wfa_ie_wpa_type = wlan_wfa_ie_wpa_type .. type_string
                    end
                end

                -- WPA2 - TKIP or AES
                if (wlan_rsn_pcs_type_f() ~= nil) then
                    local wlan_rsn_pcs_type_list = { wlan_rsn_pcs_type_f() }
                    for i, type_f in pairs(wlan_rsn_pcs_type_list) do

                        if i > 1 then
                            wlan_rsn_pcs_type = wlan_rsn_pcs_type .. ","
                        end
            
                        local type_string
                        if tostring(type_f()) == "0" then
                            type_string = "None"
                        elseif tostring(type_f())== "1" or tostring(type_f()) == "5"  then
                            type_string = "WEP"
                        elseif tostring(type_f()) == "2" then
                                type_string = "TKIP"
                        elseif tostring(type_f())== "3" or tostring(type_f()) == "4"  then
                                type_string = "AES"
                        elseif tostring(type_f()) == "6" then
                                type_string = "BIP"
                        elseif tostring(type_f()) == "7" then
                                type_string = "Group addressed traffic not allowed"
                        elseif tostring(type_f()) == "8" then
                                type_string = "GCMP"
                        else
                            type_string = tostring(type_f())
                        end
            
                        wlan_rsn_pcs_type = wlan_rsn_pcs_type .. type_string
                    end
                end
    
                -- WPA - TKIP or AES
                if (wlan_wfa_ie_wpa_ucs_type_f() ~= nil) then
                    local wlan_wfa_ie_wpa_ucs_type_list = { wlan_wfa_ie_wpa_ucs_type_f() }
                    for i, type_f in pairs(wlan_wfa_ie_wpa_ucs_type_list) do
           
                        if i > 1 then
                            wlan_wfa_ie_wpa_ucs_type = wlan_wfa_ie_wpa_ucs_type .. ","
                        end
            
                        local type_string
                        if tostring(type_f()) == "0" then
                            type_string = "None"
                        elseif tostring(type_f())== "1" or tostring(type_f()) == "5"  then
                            type_string = "WEP"
                        elseif tostring(type_f()) == "2" then
                                type_string = "TKIP"
                        elseif tostring(type_f())== "3" or tostring(type_f()) == "4"  then
                                type_string = "AES"
                        elseif tostring(type_f()) == "6" then
                                type_string = "BIP"
                        elseif tostring(type_f()) == "7" then
                                type_string = "Group addressed traffic not allowed"
                        else
                            type_string = tostring(type_f())
                        end
            
                        wlan_wfa_ie_wpa_ucs_type = wlan_wfa_ie_wpa_ucs_type .. type_string
                    end
                end
    
                local wlan_akm_type_list = ''
                local wlan_ciphersuite_list = ''
    
                wlan_akm_type_list = addToStringWithDelim(wlan_akm_type_list, wlan_rsn_akms_type, "/")
                wlan_akm_type_list = addToStringWithDelim(wlan_akm_type_list, wlan_wfa_ie_wpa_type, "/")
                if wlan_akm_type_list == '' then
                    wlan_akm_type_list = 'Open'
                end
    
                wlan_ciphersuite_list = addToStringWithDelim(wlan_ciphersuite_list, wlan_rsn_pcs_type, "/")
                wlan_ciphersuite_list = addToStringWithDelim(wlan_ciphersuite_list, wlan_wfa_ie_wpa_ucs_type, "/")
                
                if (wlan_cisco_ccx1_name_f() ~= nil) then
                    wlan_cisco_ccx1_name = tostring(wlan_cisco_ccx1_name_f())
                end
                
                if (wlan_vs_aruba_ap_name_f() ~= nil) then
                    wlan_vs_aruba_ap_name = tostring(wlan_vs_aruba_ap_name_f())
                end
                
                
                
                local fields = {    wlan_ta,
                                    wlan_ssid,
                                    wlan_ssid_hex,
                                    wlan_country_info_code,
                                    radiotap_channel_freq,
                                    wlan_ht_info_primarychannel,
                                    wlan_ht_info_secchanoffset,
                                    wlan_vht_op_channelwidth,
                                    radiotap_dbm_antsignal,
                                    wlan_ht_mcsset_rxbitmask,
                                    wlan_akm_type_list,
                                    wlan_ciphersuite_list,
                                    wlan_cisco_ccx1_name,
                                    wlan_vs_aruba_ap_name
                                }
                                
                wifi_beacon = {}
                wifi_beacon['fields'] = fields
                wifi_beacons[wlan_ta] = wifi_beacon
            end
    
            -- Display all results on 4 1st seconds, then 1 frame of every BSSID every 2 seconds
            if ((frame_time_epoch > time_epoch+2000) or (frame_time_epoch-launch_time_epoch < 4000)) then
                for wlan_ta, wifi_beacon in pairs(wifi_beacons) 
                    do 
                        local output = ""
                        local fields = wifi_beacon['fields']
                        for i, field in ipairs(fields) do
                            output = output .. field

                            if i < #fields then
                                output = output .. "\t"
                            end
                        end
                        print(output)
               
                        wifi_beacons[wlan_ta] = nil
                end

                -- Will be reset on next loop
                time_epoch=0
            end
        end -- else wlan_fc_type_subtype_f != 8
    end -- else wlan_fc_type_subtype_f is nil
end




--  wlan_rsn_akms_type
-- static const value_string ieee80211_rsn_keymgmt_vals[] = {
--     {0, "NONE"},
--     {1, "WPA"},
--     {2, "PSK"},
--     {3, "FT over IEEE 802.1X"},
--     {4, "FT using PSK"},
--     {5, "WPA (SHA256)"},
--     {6, "PSK (SHA256)"},
--     {7, "TDLS / TPK Handshake"},
--     {0, NULL}
-- };
-- static const value_string ieee80211_rsn_cipher_vals[] = {
--     {0, "NONE"},
--     {1, "WEP (40-bit)"},
--     {2, "TKIP"},
--     {3, "AES (OCB)"},
--     {4, "AES (CCM)"},
--     {5, "WEP (104-bit)"},
--     {6, "BIP"},
--     {7, "Group addressed traffic not allowed"},
--     {8, "GCMP"},
--     {0, NULL}
-- };
-- static const value_string ieee80211_wfa_ie_wpa_keymgmt_vals[] = {
--   { 0, "NONE" },
--   { 1, "WPA" },
--   { 2, "PSK" },
--   { 3, "FT over IEEE 802.1X" },
--   { 4, "FT using PSK" },
--   { 5, "WPA (SHA256)" },
--   { 6, "PSK (SHA256)" },
--   { 7, "TDLS / TPK Handshake" },
--   { 0, NULL }
-- };
-- 
-- static const value_string ieee80211_wfa_ie_wpa_cipher_vals[] = {
--   { 0, "NONE" },
--   { 1, "WEP (40-bit)" },
--   { 2, "TKIP" },
--   { 3, "AES (OCB)" },
--   { 4, "AES (CCM)" },
--   { 5, "WEP (104-bit)" },
--   { 6, "BIP" },
--   { 7, "Group addressed traffic not allowed" },
--   { 0, NULL }
-- };
